# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that parsers shoutcast files.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Parser(object):
	"""
	This class parsers a Shoutcast playlist file. The class gets the
	name of the internet radio station and the uris streams associated
	with it.
	"""

	def __init__(self, manager, playlist_file):
		"""
		Initialize object.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.

		@param manager: Reference to the manager instance.
		@type manager: A Manager object.

		@param playlist_file: The Shoutcast file to parse.
		@type playlist_file: A String object.
		"""
		self.__init_attribute(manager, playlist_file)
		self.__open_file()

	def __init_attribute(self, manager, playlist_file):
		"""
		Initialize data attributes.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param playlist_file: The Shoutcast file to parse.
		@type playlist_file: A Shoutcast object.
		"""
		self.__manager = manager
		self.__playlist_file = playlist_file
		self.__handle = None
		self.__string = ""
		self.__station_name = ""
		self.__uris = set([])
		self.__dictionary = {}
		return

	def __open_file(self):
		"""
		Asynchronously open file for parsing.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.
		"""
		try:
			from gnomevfs import make_uri_from_shell_arg
			from gnomevfs import OPEN_READ, URI
			from gnomevfs.async import open
			uri = make_uri_from_shell_arg(self.__playlist_file.strip())
			open(URI(uri), self.__open_cb, OPEN_READ, 10)
		except:
			self.__error()
		return

	def __open_cb(self, handle, result):
		"""
		Callback to the async open function.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.
		"""
		try:
			self.__handle = handle
			handle.read(4096, self.__read_cb)
		except:
			self.__error()
		return

	def __read_cb(self, handle, buffer, result, bytes):
		"""
		Callback to the async open function.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.
		"""
		from gnomevfs import EOFError
		from operator import is_, eq, contains, not_
		if is_(result, None):
			try:
				self.__string += buffer
				handle.read(4096, self.__read_cb)
			except:
				self.__error()
		elif eq(result, EOFError):
			try:
				handle.close(self.__close_cb)
			except:
				self.__error()
		else:
			try:
				handle.close(self.__close_cb)
			except:
				self.__error()
		return

	def __close_cb(self, *args):
		"""
		Callback to the async read function.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.
		"""
		self.__parse_string()
		return

	def __parse_string(self):
		"""
		Parse content of the shoutcast file.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.
		"""
		from operator import not_
		if not_(self.__string): return self.__error()
		lines = self.__string.split("\n")
		for line in lines:
			self.__parse_line(line.strip())
		if not_(self.__station_name and self.__uris): return self.__error()
		self.__dictionary[self.__station_name] = "", "", self.__uris
		self.__manager.emit("parsed", (self.__playlist_file, self.__dictionary))
		self.__destroy()
		return

	def __parse_line(self, line):
		"""
		Parse the content of a line in the Shoutcast file.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.

		@param line: A line in a Shoutcast file.
		@type line: A String object.
		"""
		if line.startswith("File"):
			string, uri = line.split("=")
			self.__uris.add(uri.strip())
			return
		if self.__station_name: return
		if line.startswith("Title"):
			try:
				string, station_name = line.split("=")
				self.__station_name = station_name.strip()
			except ValueError:
				station_list = line.split("=")[1:]
				self.__station_name = " ".join(station_list).strip()
		return

	def __error(self):
		"""
		Emit error message.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.
		"""
		if self.__handle: self.__handle.cancel()
		self.__manager.emit("parse-error", self.__playlist_file)
		self.__destroy()
		return

	def __destroy(self):
		"""
		Destroy object.

		@param self: Reference to the Parser instance.
		@type self: A Parser object.
		"""
		from Striim.utils import delete_attributes
		delete_attributes(self)
		del self
		self = None
		return
