# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates triggers for the volume
controls.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE

class Trigger(GObject):
	"""
	This class creates triggers for the volume controls and also
	initializes the volume control manager.
	"""

	__gsignals__ = {
		"destroy": (SIGNAL_RUN_LAST, TYPE_NONE, ())
	}

	def __init__(self, player):
		"""
		Initialize object.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		GObject.__init__(self)
		self.__init_attributes(player)
		self.__create_triggers()
		self.__sig_id_1 = self.connect("destroy", self.__destroy_cb)
		self.__sig_id_2 = self.__up_trigger.connect("activate", self.__up_activate_cb)
		self.__sig_id_3 = self.__down_trigger.connect("activate", self.__down_activate_cb)

	def __init_attributes(self, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__player = player
		from Manager import Manager
		self.__manager = Manager(player)
		self.__up_trigger = self.__down_trigger = None
		self.__sig_id_1 = self.__sig_id_2 = self.__sig_id_3 = None
		return

	def __create_triggers(self):
		"""
		Create triggers for volume control.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.
		"""
		from Striim.Trigger import Trigger
		self.__up_trigger = Trigger("increase_volume", "Up")
		self.__down_trigger = Trigger("decrease_volume", "Down")
		self.__player.add_triggers([self.__up_trigger, self.__down_trigger])
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.
		"""
		self.__player.remove_triggers([self.__up_trigger, self.__down_trigger])
		from Striim.utils import disconnect_signal, delete_attributes
		disconnect_signal(self.__sig_id_1, self)
		disconnect_signal(self.__sig_id_2, self.__up_trigger)
		disconnect_signal(self.__sig_id_3, self.__down_trigger)
		self.__manager.destroy()
		delete_attributes(self)
		del self
		self = None
		return

	def __up_activate_cb(self, *args):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.
		"""
		self.__manager.increase_volume()
		return

	def __down_activate_cb(self, *args):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.
		"""
		self.__manager.decrease_volume()
		return

	def destroy(self):
		"""
		Destroy object.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.
		"""
		self.emit("destroy")
		return
